@@ -76,6 +76,7 @@ static pfmlib_pmu_t *pfmlib_pmus[]=
 	&amd64_fam12h_llano_support,
 	&amd64_fam14h_bobcat_support,
 	&amd64_fam15h_interlagos_support,
+	&amd64_fam15h_nb_support,
 	&intel_core_support,
 	&intel_atom_support,
 	&intel_nhm_support,
@@ -91,6 +92,180 @@ static pfmlib_pmu_t *pfmlib_pmus[]=
 	&intel_snb_unc_cbo3_support,
 	&intel_snb_ep_support,
 	&intel_ivb_support,
+	&intel_ivb_unc_cbo0_support,
+	&intel_ivb_unc_cbo1_support,
+	&intel_ivb_unc_cbo2_support,
+	&intel_ivb_unc_cbo3_support,
+	&intel_ivb_ep_support,
+	&intel_hsw_support,
+	&intel_hsw_ep_support,
+	&intel_bdw_support,
+	&intel_bdw_ep_support,
+	&intel_skl_support,
+	&intel_rapl_support,
+	&intel_snbep_unc_cb0_support,
+	&intel_snbep_unc_cb1_support,
+	&intel_snbep_unc_cb2_support,
+	&intel_snbep_unc_cb3_support,
+	&intel_snbep_unc_cb4_support,
+	&intel_snbep_unc_cb5_support,
+	&intel_snbep_unc_cb6_support,
+	&intel_snbep_unc_cb7_support,
+	&intel_snbep_unc_ha_support,
+	&intel_snbep_unc_imc0_support,
+	&intel_snbep_unc_imc1_support,
+	&intel_snbep_unc_imc2_support,
+	&intel_snbep_unc_imc3_support,
+	&intel_snbep_unc_pcu_support,
+	&intel_snbep_unc_qpi0_support,
+	&intel_snbep_unc_qpi1_support,
+	&intel_snbep_unc_ubo_support,
+	&intel_snbep_unc_r2pcie_support,
+	&intel_snbep_unc_r3qpi0_support,
+	&intel_snbep_unc_r3qpi1_support,
+	&intel_knc_support,
+	&intel_slm_support,
+	&intel_glm_support,
+	&intel_ivbep_unc_cb0_support,
+	&intel_ivbep_unc_cb1_support,
+	&intel_ivbep_unc_cb2_support,
+	&intel_ivbep_unc_cb3_support,
+	&intel_ivbep_unc_cb4_support,
+	&intel_ivbep_unc_cb5_support,
+	&intel_ivbep_unc_cb6_support,
+	&intel_ivbep_unc_cb7_support,
+	&intel_ivbep_unc_cb8_support,
+	&intel_ivbep_unc_cb9_support,
+	&intel_ivbep_unc_cb10_support,
+	&intel_ivbep_unc_cb11_support,
+	&intel_ivbep_unc_cb12_support,
+	&intel_ivbep_unc_cb13_support,
+	&intel_ivbep_unc_cb14_support,
+	&intel_ivbep_unc_ha0_support,
+	&intel_ivbep_unc_ha1_support,
+	&intel_ivbep_unc_imc0_support,
+	&intel_ivbep_unc_imc1_support,
+	&intel_ivbep_unc_imc2_support,
+	&intel_ivbep_unc_imc3_support,
+	&intel_ivbep_unc_imc4_support,
+	&intel_ivbep_unc_imc5_support,
+	&intel_ivbep_unc_imc6_support,
+	&intel_ivbep_unc_imc7_support,
+	&intel_ivbep_unc_pcu_support,
+	&intel_ivbep_unc_qpi0_support,
+	&intel_ivbep_unc_qpi1_support,
+	&intel_ivbep_unc_qpi2_support,
+	&intel_ivbep_unc_ubo_support,
+	&intel_ivbep_unc_r2pcie_support,
+	&intel_ivbep_unc_r3qpi0_support,
+	&intel_ivbep_unc_r3qpi1_support,
+	&intel_ivbep_unc_r3qpi2_support,
+	&intel_ivbep_unc_irp_support,
+	&intel_hswep_unc_cb0_support,
+	&intel_hswep_unc_cb1_support,
+	&intel_hswep_unc_cb2_support,
+	&intel_hswep_unc_cb3_support,
+	&intel_hswep_unc_cb4_support,
+	&intel_hswep_unc_cb5_support,
+	&intel_hswep_unc_cb6_support,
+	&intel_hswep_unc_cb7_support,
+	&intel_hswep_unc_cb8_support,
+	&intel_hswep_unc_cb9_support,
+	&intel_hswep_unc_cb10_support,
+	&intel_hswep_unc_cb11_support,
+	&intel_hswep_unc_cb12_support,
+	&intel_hswep_unc_cb13_support,
+	&intel_hswep_unc_cb14_support,
+	&intel_hswep_unc_cb15_support,
+	&intel_hswep_unc_cb16_support,
+	&intel_hswep_unc_cb17_support,
+	&intel_hswep_unc_ha0_support,
+	&intel_hswep_unc_ha1_support,
+	&intel_hswep_unc_imc0_support,
+	&intel_hswep_unc_imc1_support,
+	&intel_hswep_unc_imc2_support,
+	&intel_hswep_unc_imc3_support,
+	&intel_hswep_unc_imc4_support,
+	&intel_hswep_unc_imc5_support,
+	&intel_hswep_unc_imc6_support,
+	&intel_hswep_unc_imc7_support,
+	&intel_hswep_unc_pcu_support,
+	&intel_hswep_unc_qpi0_support,
+	&intel_hswep_unc_qpi1_support,
+	&intel_hswep_unc_sb0_support,
+	&intel_hswep_unc_sb1_support,
+	&intel_hswep_unc_sb2_support,
+	&intel_hswep_unc_sb3_support,
+	&intel_hswep_unc_ubo_support,
+	&intel_hswep_unc_r2pcie_support,
+	&intel_hswep_unc_r3qpi0_support,
+	&intel_hswep_unc_r3qpi1_support,
+	&intel_hswep_unc_r3qpi2_support,
+	&intel_hswep_unc_irp_support,
+	&intel_knl_support,
+	&intel_knl_unc_imc0_support,
+	&intel_knl_unc_imc1_support,
+	&intel_knl_unc_imc2_support,
+	&intel_knl_unc_imc3_support,
+	&intel_knl_unc_imc4_support,
+	&intel_knl_unc_imc5_support,
+	&intel_knl_unc_imc_uclk0_support,
+	&intel_knl_unc_imc_uclk1_support,
+	&intel_knl_unc_edc_uclk0_support,
+	&intel_knl_unc_edc_uclk1_support,
+	&intel_knl_unc_edc_uclk2_support,
+	&intel_knl_unc_edc_uclk3_support,
+	&intel_knl_unc_edc_uclk4_support,
+	&intel_knl_unc_edc_uclk5_support,
+	&intel_knl_unc_edc_uclk6_support,
+	&intel_knl_unc_edc_uclk7_support,
+	&intel_knl_unc_edc_eclk0_support,
+	&intel_knl_unc_edc_eclk1_support,
+	&intel_knl_unc_edc_eclk2_support,
+	&intel_knl_unc_edc_eclk3_support,
+	&intel_knl_unc_edc_eclk4_support,
+	&intel_knl_unc_edc_eclk5_support,
+	&intel_knl_unc_edc_eclk6_support,
+	&intel_knl_unc_edc_eclk7_support,
+	&intel_knl_unc_cha0_support,
+	&intel_knl_unc_cha1_support,
+	&intel_knl_unc_cha2_support,
+	&intel_knl_unc_cha3_support,
+	&intel_knl_unc_cha4_support,
+	&intel_knl_unc_cha5_support,
+	&intel_knl_unc_cha6_support,
+	&intel_knl_unc_cha7_support,
+	&intel_knl_unc_cha8_support,
+	&intel_knl_unc_cha9_support,
+	&intel_knl_unc_cha10_support,
+	&intel_knl_unc_cha11_support,
+	&intel_knl_unc_cha12_support,
+	&intel_knl_unc_cha13_support,
+	&intel_knl_unc_cha14_support,
+	&intel_knl_unc_cha15_support,
+	&intel_knl_unc_cha16_support,
+	&intel_knl_unc_cha17_support,
+	&intel_knl_unc_cha18_support,
+	&intel_knl_unc_cha19_support,
+	&intel_knl_unc_cha20_support,
+	&intel_knl_unc_cha21_support,
+	&intel_knl_unc_cha22_support,
+	&intel_knl_unc_cha23_support,
+	&intel_knl_unc_cha24_support,
+	&intel_knl_unc_cha25_support,
+	&intel_knl_unc_cha26_support,
+	&intel_knl_unc_cha27_support,
+	&intel_knl_unc_cha28_support,
+	&intel_knl_unc_cha29_support,
+	&intel_knl_unc_cha30_support,
+	&intel_knl_unc_cha31_support,
+	&intel_knl_unc_cha32_support,
+	&intel_knl_unc_cha33_support,
+	&intel_knl_unc_cha34_support,
+	&intel_knl_unc_cha35_support,
+	&intel_knl_unc_cha36_support,
+	&intel_knl_unc_cha37_support,
+	&intel_knl_unc_m2pcie_support,
 	&intel_x86_arch_support, /* must always be last for x86 */
 #endif
 
@@ -110,7 +285,10 @@ static pfmlib_pmu_t *pfmlib_pmus[]=
 	&power5p_support,
 	&power6_support,
 	&power7_support,
+	&power8_support,
 	&torrent_support,
+    &powerpc_nest_mcs_read_support,
+    &powerpc_nest_mcs_write_support,
 #endif
 
 #ifdef CONFIG_PFMLIB_ARCH_SPARC
@@ -128,17 +306,35 @@ static pfmlib_pmu_t *pfmlib_pmus[]=
 #endif
 
 #ifdef CONFIG_PFMLIB_ARCH_ARM
+	&arm_cortex_a7_support,
 	&arm_cortex_a8_support,
 	&arm_cortex_a9_support,
 	&arm_cortex_a15_support,
+	&arm_1176_support,
+	&arm_qcom_krait_support,
+	&arm_cortex_a57_support,
+	&arm_cortex_a53_support,
+	&arm_xgene_support,
+#endif
+#ifdef CONFIG_PFMLIB_ARCH_ARM64
+	&arm_cortex_a57_support,
+	&arm_cortex_a53_support,
+	&arm_xgene_support,
+#endif
+
+#ifdef CONFIG_PFMLIB_ARCH_S390X
+	&s390x_cpum_cf_support,
+	&s390x_cpum_sf_support,
 #endif
 #ifdef __linux__
 	&perf_event_support,
+	&perf_event_raw_support,
 #endif
 };
 #define PFMLIB_NUM_PMUS	(int)(sizeof(pfmlib_pmus)/sizeof(pfmlib_pmu_t *))
 
 static pfmlib_os_t pfmlib_os_none;
+pfmlib_os_t *pfmlib_os = &pfmlib_os_none;
 
 static pfmlib_os_t *pfmlib_oses[]={
 	&pfmlib_os_none,
@@ -203,6 +399,61 @@ __pfm_vbprintf(const char *fmt, ...)
 	va_end(ap);
 }
 
+/*
+ * pfmlib_getl: our own equivalent to GNU getline() extension.
+ * This avoids a dependency on having a C library with
+ * support for getline().
+ */
+int
+pfmlib_getl(char **buffer, size_t *len, FILE *fp)
+{
+#define	GETL_DFL_LEN	32
+	char *b;
+	int c;
+	size_t maxsz, maxi, d, i = 0;
+
+	if (!len || !fp || !buffer)
+		return -1;
+
+	b = *buffer;
+
+	if (!b)
+		*len = 0;
+
+	maxsz = *len;
+	maxi = maxsz - 2;
+
+	while ((c = fgetc(fp)) != EOF) {
+		if (maxsz == 0 || i == maxi) {
+			if (maxsz == 0)
+				maxsz = GETL_DFL_LEN;
+			else
+				maxsz <<= 1;
+
+			if (*buffer)
+				d = &b[i] - *buffer;
+			else
+				d = 0;
+
+			*buffer = realloc(*buffer, maxsz);
+			if (!*buffer)
+				return -1;
+
+			b = *buffer + d;
+			maxi = maxsz - d - 2;
+			i = 0;
+			*len = maxsz;
+		}
+		b[i++] = c;
+		if (c == '\n')
+			break;
+	}
+	b[i] = '\0';
+	return c != EOF ? 0 : -1;
+}
+
+
+
 /*
  * append fmt+args to str such that the string is no
  * more than max characters incl. null termination
@@ -412,6 +663,10 @@ pfmlib_init_env(void)
 	str = getenv("LIBPFM_ENCODE_INACTIVE");
 	if (str)
 		pfm_cfg.inactive = 1;
+
+	str = getenv("LIBPFM_DISABLED_PMUS");
+	if (str)
+		pfm_cfg.blacklist_pmus = str;
 }
 
 static int
@@ -445,7 +700,7 @@ pfmlib_build_fstr(pfmlib_event_desc_t *e, char **fstr)
 	if (*fstr)
 		sprintf(*fstr, "%s::%s", e->pmu->name, e->fstr);
 
-	return fstr ? PFM_SUCCESS : PFM_ERR_NOMEM;
+	return *fstr ? PFM_SUCCESS : PFM_ERR_NOMEM;
 }
 
 static int
@@ -482,6 +737,35 @@ pfmlib_match_forced_pmu(const char *name)
 	return !strncasecmp(name, pfm_cfg.forced_pmu, l);
 }
 
+static int
+pfmlib_is_blacklisted_pmu(pfmlib_pmu_t *p)
+{
+	char *q, *buffer;
+	int ret = 1;
+
+	if (!pfm_cfg.blacklist_pmus)
+		return 0;
+
+	/*
+	 * scan list for matching PMU names, we accept substrings.
+	 * for instance: snbep does match snbep*
+	 */
+	buffer = strdup(pfm_cfg.blacklist_pmus);
+	if (!buffer)
+		return 0;
+
+	strcpy (buffer, pfm_cfg.blacklist_pmus);
+	for (q = strtok (buffer, ","); q != NULL; q = strtok (NULL, ",")) {
+		if (strstr (p->name, q) != NULL) {
+			goto done;
+		}
+	}
+	ret = 0;
+done:
+	free(buffer);
+	return ret;
+}
+
 static int
 pfmlib_init_pmus(void)
 {
@@ -514,6 +798,10 @@ pfmlib_init_pmus(void)
 		if (pfmlib_pmu_sanity_checks(p) != PFM_SUCCESS)
 			continue;
 
+		if (pfmlib_is_blacklisted_pmu(p)) {
+			DPRINT("%d PMU blacklisted, skipping initialization\n");
+			continue;
+		}
 		p->flags |= PFMLIB_PMU_FL_INIT;
 
 		/*
@@ -524,6 +812,17 @@ pfmlib_init_pmus(void)
 		if (ret != PFM_SUCCESS)
 			continue;
 
+		/*
+		 * check if exported by OS if needed
+		 */
+		if (p->os_detect[pfmlib_os->id]) {
+			ret = p->os_detect[pfmlib_os->id](p);
+			if (ret != PFM_SUCCESS) {
+				DPRINT("%s PMU not exported by OS\n", p->name);
+				continue;
+			}
+		}
+
 		ret = pfmlib_pmu_activate(p);
 		if (ret == PFM_SUCCESS)
 			nsuccess++;
@@ -555,9 +854,13 @@ pfmlib_init_os(void)
 		if (os->detect(os) != PFM_SUCCESS)
 			continue;
 
+		if (os != &pfmlib_os_none && pfmlib_os == &pfmlib_os_none)
+			pfmlib_os = os;
+
 		DPRINT("OS layer %s activated\n", os->name);
 		os->flags = PFMLIB_OS_FL_ACTIVATED;
 	}
+	DPRINT("default OS layer: %s\n", pfmlib_os->name);
 }
 
 int
@@ -566,27 +869,29 @@ pfm_initialize(void)
 	int ret;
 	/*
 	 * not atomic
+	 * if initialization already done, then reurn previous return value
 	 */
 	if (pfm_cfg.initdone)
-		return PFM_SUCCESS;
+		return pfm_cfg.initret;
 
 	/*
 	 * generic sanity checks
 	 */
 	if (PFM_PMU_MAX & (~PFMLIB_PMU_MASK)) {
 		DPRINT("PFM_PMU_MAX exceeds PFMLIB_PMU_MASK\n");	
-		return PFM_ERR_NOTSUPP;
+		ret = PFM_ERR_NOTSUPP;
+	} else {
+
+		pfmlib_init_env();
+
+		/* must be done before pfmlib_init_pmus() */
+		pfmlib_init_os();
+
+		ret = pfmlib_init_pmus();
 	}
 
-	pfmlib_init_env();
-
-	ret = pfmlib_init_pmus();
-	if (ret != PFM_SUCCESS)
-		return ret;
-
-	pfmlib_init_os();
-
 	pfm_cfg.initdone = 1;
+	pfm_cfg.initret  = ret;
 
 	return ret;
 }
@@ -625,8 +930,16 @@ pfm_find_event(const char *str)
 	memset(&e, 0, sizeof(e));
 
 	ret = pfmlib_parse_event(str, &e);
-	if (ret == PFM_SUCCESS)
-		return pfmlib_pidx2idx(e.pmu, e.event);
+	if (ret == PFM_SUCCESS) {
+		/*
+		 * save index so we can return it
+		 * and free the pattrs data that was
+		 * allocated in pfmlib_parse_event()
+		 */
+		ret = pfmlib_pidx2idx(e.pmu, e.event);
+
+		pfmlib_release_event(&e);
+	}
 
 	return ret;
 }
@@ -664,9 +977,10 @@ pfmlib_parse_event_attr(char *str, pfmlib_event_desc_t *d)
 	s = str;
 
 	while(s) {
-		p = strchr(s, PFMLIB_ATTR_DELIM);
-		if (p)
-			*p++ = '\0';
+	        p = s;
+	        strsep(&p, PFMLIB_ATTR_DELIM);
+		/* if (p)
+		 *p++ = '\0'; */
 
 		q = strchr(s, '=');
 		if (q)
@@ -697,10 +1011,10 @@ pfmlib_parse_event_attr(char *str, pfmlib_event_desc_t *d)
 			ainfo->name = "RAW_UMASK";
 			ainfo->type = PFM_ATTR_RAW_UMASK;
 			ainfo->ctrl = PFM_ATTR_CTRL_PMU;
-			ainfo->idx  = strtoul(s, &endptr, 0);
+			ainfo->idx  = strtoull(s, &endptr, 0);
 			ainfo->equiv= NULL;
 			if (*endptr) {
-				DPRINT("raw umask (%s) is not a number\n");
+				DPRINT("raw umask (%s) is not a number\n", s);
 				return PFM_ERR_ATTR;
 			}
 
@@ -720,6 +1034,7 @@ pfmlib_parse_event_attr(char *str, pfmlib_event_desc_t *d)
 				goto found_attr;
 			}
 		}
+		DPRINT("cannot find attribute %s\n", s);
 		return PFM_ERR_ATTR;
 found_attr:
 		type = ainfo->type;
@@ -761,7 +1076,7 @@ found_attr:
 		if (type != PFM_ATTR_UMASK && type != PFM_ATTR_RAW_UMASK && !has_val) {
 			if (type != PFM_ATTR_MOD_BOOL)
 				return PFM_ERR_ATTR_VAL;
-			has_val = 1; s = yes; /* no const */
+			s = yes; /* no const */
 			goto handle_bool;
 		}
 
@@ -783,10 +1098,6 @@ handle_bool:
 
 			endptr = NULL;
 			switch(type) {
-			case PFM_ATTR_UMASK:
-			case PFM_ATTR_RAW_UMASK:
-				/* unit mask has no value */
-				goto error;
 			case PFM_ATTR_MOD_BOOL:
 				if (strlen(s) > 1)
 					goto error;
@@ -890,11 +1201,18 @@ pfmlib_release_event(pfmlib_event_desc_t *e)
 	e->pattrs = NULL;
 }
 
+static int
+match_event(void *this, pfmlib_event_desc_t *d, const char *e, const char *s)
+{
+	return strcasecmp(e, s);
+}
+
 static int
 pfmlib_parse_equiv_event(const char *event, pfmlib_event_desc_t *d)
 {
 	pfmlib_pmu_t *pmu = d->pmu;
 	pfm_event_info_t einfo;
+	int (*match)(void *this, pfmlib_event_desc_t *d, const char *e, const char *s);
 	char *str, *s, *p;
 	int i;
 	int ret;
@@ -906,15 +1224,18 @@ pfmlib_parse_equiv_event(const char *event, pfmlib_event_desc_t *d)
 	if (!str)
 		return PFM_ERR_NOMEM;
 
-	p = strchr(s, PFMLIB_ATTR_DELIM);
-	if (p)
-		*p++ = '\0';
+	p = s;
+	strsep(&p, PFMLIB_ATTR_DELIM);
+	/* if (p)
+	 *p++ = '\0'; */
+
+	match = pmu->match_event ? pmu->match_event : match_event;
 
 	pfmlib_for_each_pmu_event(pmu, i) {
 		ret = pmu->get_event_info(pmu, i, &einfo);
 		if (ret != PFM_SUCCESS)
 			goto error;
-		if (!strcasecmp(einfo.name, s))
+		if (!match(pmu, d, einfo.name, s))
 			goto found;
 	}
 	free(str);
@@ -950,6 +1271,7 @@ pfmlib_parse_event(const char *event, pfmlib_event_desc_t *d)
 	pfm_event_info_t einfo;
 	char *str, *s, *p;
 	pfmlib_pmu_t *pmu;
+	int (*match)(void *this, pfmlib_event_desc_t *d, const char *e, const char *s);
 	const char *pname = NULL;
 	int i, j, ret;
 
@@ -978,9 +1300,11 @@ pfmlib_parse_event(const char *event, pfmlib_event_desc_t *d)
 		pname = s;
 		s = p + strlen(PFMLIB_PMU_DELIM);
 	}
-	p = strchr(s, PFMLIB_ATTR_DELIM);
-	if (p)
-		*p++ = '\0';
+	p = s;
+	strsep(&p, PFMLIB_ATTR_DELIM);
+	/* if (p)
+	 *p++ = '\0'; */
+
 	/*
 	 * for each pmu
 	 */
@@ -1003,6 +1327,8 @@ pfmlib_parse_event(const char *event, pfmlib_event_desc_t *d)
 		 */
 		if (pname && !pfmlib_pmu_active(pmu) && !pfm_cfg.inactive)
 			continue;
+
+		match = pmu->match_event ? pmu->match_event : match_event;
 		/*
 		 * for each event
 		 */
@@ -1010,7 +1336,7 @@ pfmlib_parse_event(const char *event, pfmlib_event_desc_t *d)
 			ret = pmu->get_event_info(pmu, i, &einfo);
 			if (ret != PFM_SUCCESS)
 				goto error;
-			if (!strcasecmp(einfo.name, s))
+			if (!match(pmu, d, einfo.name, s))
 				goto found;
 		}
 	}
@@ -1042,9 +1368,9 @@ found:
 	for (i = 0; i < d->nattrs; i++) {
 		pfm_event_attr_info_t *a = attr(d, i);
 		if (a->type != PFM_ATTR_RAW_UMASK)
-			DPRINT("%d %d %d %s\n", d->event, i, a->idx, d->pattrs[a->idx].name);
+			DPRINT("%d %d %"PRIu64" %s\n", d->event, i, a->idx, d->pattrs[d->attrs[i].id].name);
 		else
-			DPRINT("%d %d RAW_UMASK (0x%x)\n", d->event, i, a->idx);
+			DPRINT("%d %d RAW_UMASK (0x%"PRIx64")\n", d->event, i, a->idx);
 	}
 error:
 	free(str);
@@ -1285,6 +1611,10 @@ pfmlib_pmu_validate_encoding(pfmlib_pmu_t *pmu, FILE *fp)
 			sprintf(buf, "%s::%s:%s", pmu->name, einfo.name, ainfo.name);
 			ret = pfmlib_validate_encoding(buf, PFM_PLM3|PFM_PLM0);
 			if (ret != PFM_SUCCESS) {
+				if (pmu->can_auto_encode) {
+					if (!pmu->can_auto_encode(pmu, i, u))
+						continue;
+				}
 				/*
 				 * some PMU may not support raw encoding
 				 */
@@ -1300,6 +1630,10 @@ pfmlib_pmu_validate_encoding(pfmlib_pmu_t *pmu, FILE *fp)
 			sprintf(buf, "%s::%s", pmu->name, einfo.name);
 			ret = pfmlib_validate_encoding(buf, PFM_PLM3|PFM_PLM0);
 			if (ret != PFM_SUCCESS) {
+				if (pmu->can_auto_encode) {
+					if (!pmu->can_auto_encode(pmu, i, u))
+						continue;
+				}
 				if (ret != PFM_ERR_NOTSUPP) {
 					fprintf(fp, "cannot encode event %s : %s\n", buf, pfm_strerror(ret));
 					retval = ret;
@@ -1671,7 +2005,8 @@ pfmlib_raw_pmu_encode(void *this, const char *str, int dfl_plm, void *data)
 
 	if (!pmu->get_event_encoding[PFM_OS_NONE]) {
 		DPRINT("PMU %s does not support PFM_OS_NONE\n", pmu->name);
-		return PFM_ERR_NOTSUPP;
+		ret = PFM_ERR_NOTSUPP;
+		goto error;
 	}
 
 	ret = pmu->get_event_encoding[PFM_OS_NONE](pmu, &e);
